<?php

namespace Drupal\inline_all_css\Asset;

use Drupal\Core\Asset\AssetCollectionRendererInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Render\Markup;
use Drupal\inline_all_css\Event\CssPreRenderEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use function file_get_contents;

/**
 * Decorates the CSS collection renderer service, adds Critical CSS.
 *
 * @see \Drupal\Core\Asset\CssCollectionRenderer
 */
class CriticalCssCollectionRenderer implements AssetCollectionRendererInterface {

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a CriticalCssCollectionRenderer.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher service.
   */
  public function __construct(FileSystemInterface $file_system, EventDispatcherInterface $event_dispatcher) {
    $this->fileSystem = $file_system;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public function render(array $assets) {
    $css = '';
    foreach ($assets as $asset) {
      $file = $this->fileSystem->realpath($asset['data']);
      $css .= file_get_contents($file);
    }

    $event = new CssPreRenderEvent($css);
    /* @noinspection PhpMethodParametersCountMismatchInspection */
    $this->eventDispatcher->dispatch($event, CssPreRenderEvent::EVENT_NAME);
    $css = $event->getCss();

    return [
      [
        '#type' => 'html_tag',
        '#tag' => 'style',
        '#value' => Markup::create($css),
      ],
    ];
  }

}
