<?php

namespace Drupal\Tests\inline_all_css\Unit;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Render\Markup;
use Drupal\inline_all_css\Asset\CriticalCssCollectionRenderer;
use Drupal\inline_all_css\Event\CssPreRenderEvent;
use Drupal\Tests\UnitTestCase;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Test cases for the critical css collection renderer.
 *
 * @group inline_all_css
 */
class CssCollectionRendererTest extends UnitTestCase {

  /**
   * The subject under test.
   *
   * @var \Drupal\inline_all_css\Asset\CriticalCssCollectionRenderer
   */
  protected $instance;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $filesystem = $this->getMockBuilder(FileSystemInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $filesystem
      ->method('realpath')
      ->willReturnMap([
        ['public://test-1.css', __DIR__ . '/../../fixtures/test-1.css'],
        ['public://test-2.css', __DIR__ . '/../../fixtures/test-2.css'],
        ['public://test-3.css', __DIR__ . '/../../fixtures/test-3.css'],
      ]);

    $eventDispatcher = $this->getMockBuilder(EventDispatcherInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $eventDispatcher->method('dispatch')
      ->willReturnCallback(static function (CssPreRenderEvent $event) {
        $css = $event->getCss();
        $css .= <<<CSS
.event {
  color: gray;
}
CSS;
        $event->setCss($css);
      });

    $this->instance = new CriticalCssCollectionRenderer($filesystem, $eventDispatcher);
  }

  /**
   * Test case for inline CSS rendering.
   */
  public function testCssCollectionRenderer() {
    $assets = [
      ['data' => 'public://test-1.css'],
      ['data' => 'public://test-2.css'],
      ['data' => 'public://test-3.css'],
    ];

    $expected = [
      [
        '#type' => 'html_tag',
        '#tag' => 'style',
        '#value' => Markup::create(<<<CSS
* {
  color: red;
}
html {
  color: green;
}
body {
  color: blue;
}
.event {
  color: gray;
}
CSS),
      ],
    ];
    $actual = $this->instance->render($assets);

    static::assertEquals($expected, $actual);
  }

}
